import json
from flask import Flask, redirect, render_template, url_for
from decorators import admin
from models import db
from strava import Strava
from stravalib import unithelper

app = Flask(__name__, instance_relative_config=True)
app.config.from_object('config')
app.config.from_pyfile('config.py')

db.init_app(app)

@app.route('/')
def index():
    return render_template('index.html')

@admin
@app.route('/strava_auth')
def strava_auth():
    if Strava.check_auth():
        return Strava.deauthorize()
    else:
        return Strava.authorize()

@app.route('/api/<api_name>')
def api(api_name):
    if api_name == "strava":
        s = Strava()
        return json.dumps([{
            'name': a.name,
            'type': a.type,
            'moving_time': a.moving_time.seconds,
            'date': str(a.start_date),
            'distance': unithelper.miles(a.distance).num,
            'athlete_id': a.athlete.id,
            'url': 'https://strava.com/activity/{}'.format(a.id)
            } for a in s.activities])

@app.context_processor
def cp_is_admin():
    from util import is_admin
    return dict(is_admin=is_admin)

@app.context_processor
def cp_strava():
    strava=Strava.check_auth()
    return dict(strava=strava)
