from flask import request, redirect, current_app
from stravalib.client import Client
from requests.exceptions import HTTPError
from sqlalchemy.orm.exc import NoResultFound
from models import db, StravaApiToken

class Strava(Client):
    """Wrapper for stavalib Client, which is a Strava API client.
    Grabs my API token from database to authenticate."""

    def __init__(self):
        token = db.session.query(StravaApiToken).one().token
        super(Strava, self).__init__(token)

    @classmethod
    def check_auth(cls, token = None):
        if not token:
            try:
                token = db.session.query(StravaApiToken).one().token
            except NoResultFound:
                return False
        strava = Client(token)
        try:
            return strava.get_athlete().email
        except (HTTPError, AttributeError):
            return False

    @classmethod
    def authorize(cls, code = None):
        """[admin] Redirect to Strava to get an access token."""
        code = code or request.args.get('code', None)
        if not code:
            strava = Client()
            source_url = request.referrer
            authorized_url = request.url
            authorize_url = strava.authorization_url(
                client_id=current_app.config['STRAVA_APP_ID'],
                redirect_uri=authorized_url,
                state=source_url)
            return redirect(authorize_url)
        else:
            source_url = request.args.get('state', None)
            strava = Client()
            access_token = strava.exchange_code_for_token(
                client_id=current_app.config['STRAVA_APP_ID'],
                client_secret=current_app.config['STRAVA_APP_SECRET'],
                code=code)
            strava.access_token = access_token
            db.session.add(StravaApiToken(token=access_token))
            db.session.commit()
            if source_url:
                return redirect(source_url)
            return "Auth success: {}".format(strava.get_athlete().email)

    @classmethod
    def deauthorize(cls):
        """[admin] Remove stored access token."""
        token = db.session.query(StravaApiToken).one()
        db.session.delete(token)
        db.session.commit()
        return redirect(request.referrer) if request.referrer else "Deleted Strava auth token."

    @property
    def activities(self):
        """Get list of activities from Strava API."""
        return self.get_activities()
